@@ -6,6 +6,11 @@
 
 #pragma once
 
+#include "assembly_graph/core/graph.hpp"
+
+#include <string>
+#include <vector>
+
 namespace debruijn_graph {
 
 // The stuff is template here to provide interface w/o including any headers
@@ -15,11 +20,14 @@ namespace debruijn_graph {
 struct EdgeIndexRefiller {
     std::string workdir_;
 
-    EdgeIndexRefiller(const std::string &workdir)
-            : workdir_(workdir) {}
+    EdgeIndexRefiller(const std::string &workdir);
 
-    template<class EdgeIndex, class Graph>
+    template<class EdgeIndex>
     void Refill(EdgeIndex &index, const Graph &g);
+
+    template<class EdgeIndex>
+    void Refill(EdgeIndex &index, const Graph &g,
+                const std::vector<typename Graph::EdgeId> &edges);
 };
 
 }
