@@ -5,24 +5,19 @@
 //* See file LICENSE for details.
 //***************************************************************************
 
-/*
- * extension.hpp
- *
- *  Created on: Mar 5, 2012
- *      Author: andrey
- */
-
 #ifndef EXTENSION_HPP_
 #define EXTENSION_HPP_
 
+#include "weight_counter.hpp"
+#include "pe_utils.hpp"
+#include "assembly_graph/components/graph_component.hpp"
+#include "modules/alignment/rna/ss_coverage.hpp"
+
+#include <queue>
 #include <cfloat>
 #include <iostream>
 #include <fstream>
 #include <map>
-#include "weight_counter.hpp"
-#include "pe_utils.hpp"
-
-//#include "scaff_supplementary.hpp"
 
 namespace path_extend {
 
@@ -62,10 +57,8 @@ protected:
     }
 
     virtual int ExcludeTrivialWithBulges(const BidirectionalPath& path, std::set<size_t>& edges) const {
-
-        if (path.Empty()) {
+        if (path.Empty())
             return 0;
-        }
 
         int lastEdge = (int) path.Size() - 1;
         do {
@@ -75,18 +68,17 @@ protected:
             if (lastEdge >= 0) {
                 VertexId v = g_.EdgeEnd(path[lastEdge]);
                 VertexId u = g_.EdgeStart(path[lastEdge]);
-                auto bulgeCandidates = g_.IncomingEdges(v);
 
-                for (const auto& candidate: bulgeCandidates) {
-                    if (g_.EdgeStart(candidate) != u) {
+                for (EdgeId e : g_.IncomingEdges(v)) {
+                    if (g_.EdgeStart(e) != u) {
                         bulge = false;
                         break;
                     }
                 }
 
-                if (!bulge) {
+                if (!bulge)
                     break;
-                }
+
                 --lastEdge;
             }
         } while (lastEdge >= 0);
@@ -102,8 +94,8 @@ protected:
 class PreserveSimplePathsAnalyzer: public PathAnalyzer {
 
 public:
-    PreserveSimplePathsAnalyzer(const Graph &g) : PathAnalyzer(g) {
-    }
+    PreserveSimplePathsAnalyzer(const Graph &g)
+            : PathAnalyzer(g) { }
 
     int ExcludeTrivial(const BidirectionalPath& path, std::set<size_t>& edges, int from = -1) const override {
         int edgeIndex = PathAnalyzer::ExcludeTrivial(path, edges, from);
@@ -117,10 +109,8 @@ public:
     }
 
     int ExcludeTrivialWithBulges(const BidirectionalPath& path, std::set<size_t>& edges) const override {
-
-        if (path.Empty()) {
+        if (path.Empty())
             return 0;
-        }
 
         int lastEdge = (int) path.Size() - 1;
         bool has_bulge = false;
@@ -130,16 +120,14 @@ public:
             if (lastEdge >= 0) {
                 VertexId v = g_.EdgeEnd(path[lastEdge]);
                 VertexId u = g_.EdgeStart(path[lastEdge]);
-                auto bulgeCandidates = g_.IncomingEdges(v);
                 has_bulge = true;
 
-                for (auto iter = bulgeCandidates.begin(); iter != bulgeCandidates.end(); ++iter) {
-                    if (g_.EdgeStart(*iter) != u) {
+                for (EdgeId e : g_.IncomingEdges(v)) {
+                    if (g_.EdgeStart(e) != u) {
                         has_bulge = false;
                         break;
                     }
                 }
-
                 --lastEdge;
             }
         } while (lastEdge >= 0);
@@ -180,15 +168,15 @@ public:
 
 protected:
     const Graph& g_;
-    shared_ptr<WeightCounter> wc_;
+    std::shared_ptr<WeightCounter> wc_;
     //FIXME memory leak?!
     std::vector<ExtensionChooserListener *> listeners_;
 
     double weight_threshold_;
 
 public:
-    ExtensionChooser(const Graph& g, shared_ptr<WeightCounter> wc = nullptr, double weight_threshold = -1.):
-        g_(g), wc_(wc), 
+    ExtensionChooser(const Graph &g, std::shared_ptr<WeightCounter> wc = nullptr, double weight_threshold = -1.):
+        g_(g), wc_(wc),
         weight_threshold_(weight_threshold) {
     }
 
@@ -218,11 +206,11 @@ public:
         }
     }
 
-    bool WeightCounterBased() const {
+    bool WeightCounterBased() const noexcept {
         return wc_ != nullptr;
     }
 
-    shared_ptr<WeightCounter> wc() const {
+    std::shared_ptr<WeightCounter> wc() const {
         return wc_;
     }
 
@@ -249,13 +237,13 @@ private:
 
 
 class JointExtensionChooser: public ExtensionChooser {
-    shared_ptr<ExtensionChooser> first_;
-    shared_ptr<ExtensionChooser> second_;
+    std::shared_ptr<ExtensionChooser> first_;
+    std::shared_ptr<ExtensionChooser> second_;
 
 public:
     JointExtensionChooser(const Graph& g,
-                          shared_ptr<ExtensionChooser> first,
-                          shared_ptr<ExtensionChooser> second): ExtensionChooser(g),
+                          std::shared_ptr<ExtensionChooser> first,
+                          std::shared_ptr<ExtensionChooser> second): ExtensionChooser(g),
         first_(first), second_(second) {
     }
 
@@ -263,8 +251,8 @@ public:
         EdgeContainer answer;
         auto r1 = first_->Filter(path, edges);
         auto r2 = second_->Filter(path, edges);
-        for (auto ewd1 : r1) {
-            for (auto ewd2 : r2) {
+        for (const auto& ewd1 : r1) {
+            for (const auto& ewd2 : r2) {
                 if (ewd1.e_ == ewd2.e_) {
                     VERIFY(ewd1.d_ == ewd2.d_);
                     answer.push_back(ewd1);
@@ -417,14 +405,14 @@ class ExcludingExtensionChooser: public ExtensionChooser {
         AlternativeContainer weights;
         for (auto iter = edges.begin(); iter != edges.end(); ++iter) {
             double weight = wc_->CountWeight(path, iter->e_, to_exclude);
-            weights.insert(std::make_pair(weight, *iter));
+            weights.emplace(weight, *iter);
             DEBUG("Candidate " << g_.int_id(iter->e_) << " weight " << weight << " length " << g_.length(iter->e_));
         }
         NotifyAll(weights);
         return weights;
     }
 
-    EdgeContainer FindPossibleEdges(const AlternativeContainer& weights, 
+    EdgeContainer FindPossibleEdges(const AlternativeContainer& weights,
             double max_weight) const {
         EdgeContainer top;
         auto possible_edge = weights.lower_bound(max_weight / prior_coeff_);
@@ -458,7 +446,8 @@ protected:
 
 
 public:
-    ExcludingExtensionChooser(const Graph& g, shared_ptr<WeightCounter> wc, PathAnalyzer analyzer, double weight_threshold, double priority) :
+    ExcludingExtensionChooser(const Graph &g, std::shared_ptr<WeightCounter> wc, PathAnalyzer analyzer,
+                              double weight_threshold, double priority) :
             ExtensionChooser(g, wc, weight_threshold), analyzer_(analyzer), prior_coeff_(priority) {
 
     }
@@ -514,9 +503,9 @@ protected:
 
             index--;
         }
-        
+
         //excluding based on presense of ambiguous paired info
-        map<size_t, unsigned> edge_2_extension_cnt;
+        std::map<size_t, unsigned> edge_2_extension_cnt;
         for (size_t i = 0; i < edges.size(); ++i) {
             for (size_t e : wc_->PairInfoExist(path, edges.at(i).e_)) {
                 edge_2_extension_cnt[e] += 1;
@@ -533,7 +522,7 @@ protected:
 
 public:
 
-    SimpleExtensionChooser(const Graph& g, shared_ptr<WeightCounter> wc, double weight_threshold, double priority) :
+    SimpleExtensionChooser(const Graph &g, std::shared_ptr<WeightCounter> wc, double weight_threshold, double priority) :
         ExcludingExtensionChooser(g, wc, PathAnalyzer(g), weight_threshold, priority) {
     }
 
@@ -569,7 +558,7 @@ protected:
 public:
 
     IdealBasedExtensionChooser(const Graph &g,
-                               shared_ptr<WeightCounter> wc,
+                               std::shared_ptr<WeightCounter> wc,
                                double weight_threshold,
                                double priority) :
         ExcludingExtensionChooser(g, wc, PathAnalyzer(g), weight_threshold, priority) {
@@ -601,7 +590,7 @@ protected:
 
 public:
 
-    RNAExtensionChooser(const Graph& g, shared_ptr<WeightCounter> wc, double weight_threshold, double priority) :
+    RNAExtensionChooser(const Graph &g, std::shared_ptr<WeightCounter> wc, double weight_threshold, double priority) :
         ExcludingExtensionChooser(g, wc, PreserveSimplePathsAnalyzer(g), weight_threshold, priority) {
     }
 
@@ -630,29 +619,30 @@ protected:
         }
     }
 public:
-    LongEdgeExtensionChooser(const Graph& g, shared_ptr<WeightCounter> wc, double weight_threshold, double priority) :
+    LongEdgeExtensionChooser(const Graph& g, std::shared_ptr<WeightCounter> wc, double weight_threshold, double priority) :
         ExcludingExtensionChooser(g, wc, PathAnalyzer(g), weight_threshold, priority) {
     }
 };
 
 class ScaffoldingExtensionChooser : public ExtensionChooser {
     typedef ExtensionChooser base;
+    typedef std::vector<std::pair<int, double>> Histogram;
     double raw_weight_threshold_;
     double cl_weight_threshold_;
     const double is_scatter_coeff_ = 3.0;
     double relative_cutoff_;
 
-    void AddInfoFromEdge(const std::vector<int>& distances, const std::vector<double>& weights, 
-                         std::vector<pair<int, double>>& histogram, size_t len_to_path_end) const {
+    void AddInfoFromEdge(const std::vector<int> &distances, const std::vector<double> &weights,
+                         Histogram &histogram, size_t len_to_path_end) const {
         for (size_t l = 0; l < distances.size(); ++l) {
             //todo commented out condition seems unnecessary and should be library dependent! do we need "max(0" there?
             if (/*distances[l] > max(0, (int) len_to_path_end - int(1000)) && */math::ge(weights[l], raw_weight_threshold_)) {
-                histogram.push_back(make_pair(distances[l] - (int) len_to_path_end, weights[l]));
+                histogram.emplace_back(distances[l] - (int) len_to_path_end, weights[l]);
             }
         }
     }
 
-    int CountMean(const vector<pair<int, double> >& histogram) const {
+    int CountMean(const Histogram &histogram) const {
         double dist = 0.0;
         double sum = 0.0;
         for (size_t i = 0; i < histogram.size(); ++i) {
@@ -663,12 +653,11 @@ class ScaffoldingExtensionChooser : public ExtensionChooser {
         return (int) round(dist);
     }
 
-    void GetDistances(EdgeId e1, EdgeId e2, std::vector<int>& dist,
-            std::vector<double>& w) const {
+    void GetDistances(EdgeId e1, EdgeId e2, std::vector<int> &dist, std::vector<double> &w) const {
         wc_->PairedLibrary().CountDistances(e1, e2, dist, w);
     }
 
-    void CountAvrgDists(const BidirectionalPath& path, EdgeId e, std::vector<pair<int, double>> & histogram) const {
+    void CountAvrgDists(const BidirectionalPath &path, EdgeId e, Histogram &histogram) const {
         for (size_t j = 0; j < path.Size(); ++j) {
             std::vector<int> distances;
             std::vector<double> weights;
@@ -679,12 +668,12 @@ class ScaffoldingExtensionChooser : public ExtensionChooser {
         }
     }
 
-
-
-    void FindBestFittedEdgesForClustered(const BidirectionalPath& path, const set<EdgeId>& edges, EdgeContainer& result) const {
+    void FindBestFittedEdgesForClustered(const BidirectionalPath &path,
+                                         const std::set<EdgeId>& edges,
+                                         EdgeContainer& result) const {
         for (EdgeId e : edges) {
+            Histogram histogram;
             DEBUG("Analyzing edge " << g_.int_id(e))
-            std::vector<pair<int, double>> histogram;
             CountAvrgDists(path, e, histogram);
             double sum = 0.0;
             for (size_t j = 0; j < histogram.size(); ++j) {
@@ -717,13 +706,13 @@ class ScaffoldingExtensionChooser : public ExtensionChooser {
         return g_.IncomingEdgeCount(g_.EdgeStart(e)) == 0;
     }
 
-    set<EdgeId> FindCandidates(const BidirectionalPath& path) const {
-        set<EdgeId> jumping_edges;
+    std::set<EdgeId> FindCandidates(const BidirectionalPath &path) const {
+        std::set<EdgeId> jumping_edges;
         const auto& lib = wc_->PairedLibrary();
         //todo lib (and FindJumpEdges) knows its var so it can be counted there
         int is_scatter = int(math::round(lib.GetIsVar() * is_scatter_coeff_));
         for (int i = (int) path.Size() - 1; i >= 0 && path.LengthAt(i) - g_.length(path.At(i)) <= lib.GetISMax(); --i) {
-            set<EdgeId> jump_edges_i;
+            std::set<EdgeId> jump_edges_i;
             lib.FindJumpEdges(path.At(i), jump_edges_i,
                                std::max(0, (int)path.LengthAt(i) - is_scatter),
                                //FIXME do we need is_scatter here?
@@ -739,9 +728,7 @@ class ScaffoldingExtensionChooser : public ExtensionChooser {
     }
 
 public:
-
-
-    ScaffoldingExtensionChooser(const Graph& g, shared_ptr<WeightCounter> wc,
+    ScaffoldingExtensionChooser(const Graph& g, std::shared_ptr<WeightCounter> wc,
                                 double cl_weight_threshold,
                                 double is_scatter_coeff,
                                 double relative_cutoff = 0.0) :
@@ -755,7 +742,7 @@ public:
         if (edges.empty()) {
             return edges;
         }
-        set<EdgeId> candidates = FindCandidates(path);
+        auto candidates = FindCandidates(path);
         for (auto e : path) {
             if (candidates.find(e) != candidates.end()) {
                 DEBUG(g_.int_id(e) << " is removed due to presence in the path")
@@ -785,8 +772,9 @@ private:
     DECL_LOGGER("ScaffoldingExtensionChooser");
 };
 
-inline bool EdgeWithWeightCompareReverse(const pair<EdgeId, double>& p1,
-                                      const pair<EdgeId, double>& p2) {
+template <class T>
+inline bool EdgeWithWeightCompareReverse(const std::pair<T, double>& p1,
+                                         const std::pair<T, double>& p2) {
     return p1.second > p2.second;
 }
 
@@ -860,10 +848,9 @@ private:
         return true;
     }
 
-    bool CheckInconsistence(
-            const BidirectionalPath& path1, size_t pos1,
-            const BidirectionalPath& path2, size_t pos2,
-            const BidirectionalPathSet& cov_paths) const {
+    bool CheckInconsistence(const BidirectionalPath& path1, size_t pos1,
+                            const BidirectionalPath& path2, size_t pos2,
+                            const BidirectionalPathSet& cov_paths) const {
         size_t first_diff_pos1 = FirstNotEqualPosition(path1, pos1, path2, pos2, false);
         size_t first_diff_pos2 = FirstNotEqualPosition(path2, pos2, path1, pos1, false);
         if (first_diff_pos1 != -1UL && first_diff_pos2 != -1UL) {
@@ -897,13 +884,12 @@ private:
         return false;
     }
 
-    std::pair<double, double> GetSubPathsWeights(
-            const BidirectionalPath& cand1, const BidirectionalPath& cand2,
-            const BidirectionalPathSet& cov_paths) const {
+    std::pair<double, double> GetSubPathsWeights(const BidirectionalPath& cand1,
+                                                 const BidirectionalPath& cand2,
+                                                 const BidirectionalPathSet& cov_paths) const {
         double weight1 = 0.0;
         double weight2 = 0.0;
-        for (auto iter = cov_paths.begin(); iter != cov_paths.end(); ++iter) {
-            BidirectionalPath* path = *iter;
+        for (const BidirectionalPath *path : cov_paths) {
             if (ContainSubPath(*path, cand1)) {
                 weight1 += path->GetWeight();
             } else if (ContainSubPath(*path, cand2)) {
@@ -927,23 +913,24 @@ private:
        double sum_cov = 0;
        size_t sum_len = 0;
        size_t total_len = 0;
-       for (auto iter = g_.ConstEdgeBegin(); !iter.IsEnd(); ++iter) {
-           total_len += g_.length(*iter);
-           if (g_.length(*iter) >= max_repeat_length_) {
-               sum_cov += g_.coverage(*iter) * (double)g_.length(*iter);
-               sum_len += g_.length(*iter);
+       for (EdgeId e : g_.edges()) {
+           total_len += g_.length(e);
+           if (g_.length(e) >= max_repeat_length_) {
+               sum_cov += g_.coverage(e) * (double) g_.length(e);
+               sum_len += g_.length(e);
            }
        }
        if (sum_len * 4 < total_len) return;
        sum_cov /= (double)sum_len;
        DEBUG("average coverage of long edges: " << sum_cov) ;
-       for (auto iter = g_.ConstEdgeBegin(); !iter.IsEnd(); ++iter) {
-           if (g_.length(*iter) > 500 && (double)g_.coverage(*iter) < 1.2 * sum_cov) {
-               if (unique_edges_.find(*iter) == unique_edges_.end()) {
-                   unique_edges_.insert(*iter);
-                   unique_edges_.insert(g_.conjugate(*iter));
-                   DEBUG("Added coverage based unique edge " << g_.int_id(*iter) << " len "<< g_.length(*iter) << " " << g_.coverage(*iter));
-               }
+       for (EdgeId e : g_.edges()) {
+           if (g_.length(e) > 500 && (double)g_.coverage(e) < 1.2 * sum_cov) {
+               if (unique_edges_.count(e))
+                   continue;
+
+               unique_edges_.insert(e);
+               unique_edges_.insert(g_.conjugate(e));
+               DEBUG("Added coverage based unique edge " << g_.int_id(e) << " len "<< g_.length(e) << " " << g_.coverage(e));
            }
        }
    }
@@ -951,11 +938,12 @@ private:
 
     void FindAllUniqueEdges() {
         DEBUG("Looking for unique edges");
-        for (auto iter = g_.ConstEdgeBegin(); !iter.IsEnd(); ++iter) {
-            if (UniqueEdge(*iter)) {
-                unique_edges_.insert(*iter);
-                unique_edges_.insert(g_.conjugate(*iter));
-            }
+        for (EdgeId e : g_.edges()) {
+            if (!UniqueEdge(e))
+                continue;
+
+            unique_edges_.insert(e);
+            unique_edges_.insert(g_.conjugate(e));
         }
         DEBUG("coverage based uniqueness started");
         if (!uneven_depth_)
@@ -972,51 +960,106 @@ private:
     bool uneven_depth_;
 };
 
-//class SimpleScaffolding {
-//public:
-//    SimpleScaffolding(const Graph& g) : g_(g) {}
-//
-//    BidirectionalPath FindMaxCommonPath(const vector<BidirectionalPath*>& paths,
-//                                        size_t max_diff_len) const {
-//        BidirectionalPath max_end(g_);
-//        for (auto it1 = paths.begin(); it1 != paths.end(); ++it1) {
-//            BidirectionalPath* p1 = *it1;
-//            for (size_t i = 0; i < p1->Size(); ++i) {
-//                if (p1->Length() - p1->LengthAt(i) > max_diff_len) {
-//                    break;
-//                }
-//                bool contain_all = true;
-//                for (size_t i1 = i + 1; i1 <= p1->Size() && contain_all; ++i1) {
-//                    BidirectionalPath subpath = p1->SubPath(i, i1);
-//                    for (auto it2 = paths.begin();  it2 != paths.end() && contain_all; ++it2) {
-//                        BidirectionalPath* p2 = *it2;
-//                        vector<size_t> positions2 = p2->FindAll(subpath.At(0));
-//                        bool contain = false;
-//                        for (size_t ipos2 = 0; ipos2 < positions2.size(); ++ipos2) {
-//                            size_t pos2 = positions2[ipos2];
-//                            if (p2->Length() - p2->LengthAt(pos2) <= max_diff_len
-//                                    && EqualEnds(subpath, 0, *p2, pos2, false)) {
-//                                contain = true;
-//                                break;
-//                            }
-//                        }
-//                        if (!contain) {
-//                            contain_all = false;
-//                        }
-//                    }
-//                    if (contain_all && (i1 - i) >= max_end.Size()) {
-//                        max_end.Clear();
-//                        max_end.PushBack(subpath);
-//                    }
-//                }
-//            }
-//        }
-//        return max_end;
-//    }
-//
-//private:
-//    const Graph& g_;
-//};
+class LongReadsRNAExtensionChooser : public ExtensionChooser {
+public:
+    LongReadsRNAExtensionChooser(const Graph& g,
+                                 const GraphCoverageMap& read_paths_cov_map,
+                                 double filtering_threshold,
+                                 size_t min_significant_overlap)
+        : ExtensionChooser(g),
+          filtering_threshold_(filtering_threshold),
+          min_significant_overlap_(min_significant_overlap),
+          cov_map_(read_paths_cov_map)
+    {
+        DEBUG("Created LongReadsRNAExtensionChooser with params: filtering_threshold_ = " << filtering_threshold_ << ", min_significant_overlap_ = " << min_significant_overlap_);
+    }
+
+    /* Choose extension as correct only if we have reads that traverse a unique edge from the path and this extension.
+     * Edge is unique if all reads mapped to this edge are consistent.
+     * Two reads are consistent if they can form one path in the graph.
+     */
+    EdgeContainer Filter(const BidirectionalPath& path,
+                         const EdgeContainer& edges) const override {
+        if (edges.empty()) {
+            return edges;
+        }
+        DEBUG("We are in Filter of LongReadsRNAExtensionChooser");
+        path.PrintDEBUG();
+
+        std::map<EdgeId, double> weights_candidates;
+        DEBUG("Candidates")
+        for (auto it = edges.begin(); it != edges.end(); ++it) {
+            DEBUG(g_.int_id(it->e_));
+            weights_candidates.emplace(it->e_, 0.0);
+        }
+
+        std::set<EdgeId> filtered_candidates;
+        auto support_paths = cov_map_.GetCoveringPaths(path.Back());
+        DEBUG("Found " << support_paths.size() << " supporting paths");
+        for (auto it = support_paths.begin(); it != support_paths.end(); ++it) {
+            auto supporting_path = *it;
+            auto positions = supporting_path->FindAll(path.Back());
+
+            for (size_t i = 0; i < positions.size(); ++i) {
+                if ((int) positions[i] < (int) supporting_path->Size() - 1
+                    && EqualBegins(path, path.Size() - 1, *supporting_path, positions[i], false)) {
+                    DEBUG("Supporting path matches, Checking unique path_back for " << supporting_path->GetId());
+
+                    if (UniqueBackPath(*supporting_path, positions[i])) {
+                        DEBUG("Success");
+
+                        EdgeId next = supporting_path->At(positions[i] + 1);
+                        weights_candidates[next] += supporting_path->GetWeight();
+                        filtered_candidates.insert(next);
+                    }
+                }
+            }
+        }
+        DEBUG("Supported candidates");
+        for (auto iter = weights_candidates.begin(); iter != weights_candidates.end(); ++iter) {
+            DEBUG("Candidate " << g_.int_id(iter->first) << " weight " << iter->second);
+        }
+        std::vector<std::pair<EdgeId, double> > sort_res = MapToSortVector(weights_candidates);
+        if (sort_res.size() < 1 || sort_res[0].second < filtering_threshold_) {
+            filtered_candidates.clear();
+        }
+        EdgeContainer result;
+        for (auto it = edges.begin(); it != edges.end(); ++it) {
+            if (filtered_candidates.find(it->e_) != filtered_candidates.end()) {
+                result.push_back(*it);
+                DEBUG("Inserting to results: " << g_.int_id(it->e_));
+            }
+        }
+        if (result.size() == 0) {
+            DEBUG("Long reads don't help =(");
+        }
+        return result;
+    }
+
+private:
+
+    bool UniqueBackPath(const BidirectionalPath& path, size_t pos) const {
+        int int_pos = (int) pos;
+        while (int_pos >= 0) {
+            if (g_.length(path.At(int_pos)) >= min_significant_overlap_)
+                return true;
+            int_pos--;
+        }
+        return false;
+    }
+
+    std::vector<std::pair<EdgeId, double> > MapToSortVector(const std::map<EdgeId, double>& map) const {
+        std::vector<std::pair<EdgeId, double> > result(map.begin(), map.end());
+        std::sort(result.begin(), result.end(), EdgeWithWeightCompareReverse<EdgeId>);
+        return result;
+    }
+
+    double filtering_threshold_;
+    size_t min_significant_overlap_;
+    const GraphCoverageMap& cov_map_;
+
+    DECL_LOGGER("LongReadsRNAExtensionChooser");
+};
 
 class LongReadsExtensionChooser : public ExtensionChooser {
 public:
@@ -1043,17 +1086,17 @@ public:
      * Edge is unique if all reads mapped to this edge are consistent.
      * Two reads are consistent if they can form one path in the graph.
      */
-    EdgeContainer Filter(const BidirectionalPath& path,
-                                 const EdgeContainer& edges) const override {
+    EdgeContainer Filter(const BidirectionalPath &path, const EdgeContainer &edges) const override {
         if (edges.empty()) {
             return edges;
-        }DEBUG("We in Filter of LongReadsExtensionChooser");
-        path.PrintDEBUG();
-        map<EdgeId, double> weights_cands;
-        for (auto it = edges.begin(); it != edges.end(); ++it) {
-            weights_cands.insert(make_pair(it->e_, 0.0));
         }
-        set<EdgeId> filtered_cands;
+        DEBUG("We are in Filter of LongReadsExtensionChooser");
+        path.PrintDEBUG();
+        std::map<EdgeId, double> weights_cands;
+        for (const auto &edge : edges) {
+            weights_cands.emplace(edge.e_, 0.0);
+        }
+        std::set<EdgeId> filtered_cands;
         auto support_paths = cov_map_.GetCoveringPaths(path.Back());
         DEBUG("Found " << support_paths.size() << " covering paths!!!");
         for (auto it = support_paths.begin(); it != support_paths.end(); ++it) {
@@ -1078,7 +1121,7 @@ public:
         for (auto iter = weights_cands.begin(); iter != weights_cands.end(); ++iter) {
             DEBUG("Candidate " << g_.int_id(iter->first) << " weight " << iter->second);
         }
-        vector<pair<EdgeId, double> > sort_res = MapToSortVector(weights_cands);
+        auto sort_res = MapToSortVector(weights_cands);
         DEBUG("sort res " << sort_res.size() << " tr " << weight_priority_threshold_);
         if (sort_res.size() < 1 || sort_res[0].second < filtering_threshold_) {
             filtered_cands.clear();
@@ -1117,9 +1160,9 @@ private:
         return false;
     }
 
-    vector<pair<EdgeId, double> > MapToSortVector(const map<EdgeId, double>& map) const {
-        vector<pair<EdgeId, double> > result(map.begin(), map.end());
-        std::sort(result.begin(), result.end(), EdgeWithWeightCompareReverse);
+    std::vector<std::pair<EdgeId, double>> MapToSortVector(const std::map<EdgeId, double>& map) const {
+        std::vector<std::pair<EdgeId, double>> result(map.begin(), map.end());
+        std::sort(result.begin(), result.end(), EdgeWithWeightCompareReverse<EdgeId>);
         return result;
     }
 
@@ -1132,13 +1175,178 @@ private:
     DECL_LOGGER("LongReadsExtensionChooser");
 };
 
+class TrustedContigsExtensionChooser : public ExtensionChooser {
+public:
+    TrustedContigsExtensionChooser(const Graph& g,
+                                    const GraphCoverageMap& read_paths_cov_map,
+                                    double filtering_threshold,
+                                    double weight_priority_threshold,
+                                    double unique_edge_priority_threshold,
+                                    size_t min_significant_overlap,
+                                    size_t max_repeat_length,
+                                    bool uneven_depth,
+                                    bool use_low_quality_matching = false)
+            : ExtensionChooser(g)
+            , filtering_threshold_(filtering_threshold)
+            , weight_priority_threshold_(weight_priority_threshold)
+            , min_significant_overlap_(min_significant_overlap)
+            , cov_map_(read_paths_cov_map)
+            , unique_edge_analyzer_(g, cov_map_, filtering_threshold,
+                                    unique_edge_priority_threshold,
+                                    max_repeat_length, uneven_depth)
+            , use_low_quality_matching_(use_low_quality_matching)
+    {}
+
+    /// @returns the possible next edge of the path
+    EdgeContainer Filter(const BidirectionalPath &path, const EdgeContainer &) const override {
+        DEBUG("We are in Filter of TrustedContigsExtensionChooser");
+        path.PrintDEBUG();
+
+        std::map<EdgeWithDistance, double> weights_cands;
+        auto filtered_cands = GetHighQualityCandidats(path, weights_cands);
+        if (use_low_quality_matching_ && filtered_cands.empty())
+            filtered_cands = GetLowQualityCandidats(path, weights_cands);
+
+        DEBUG("Candidates:");
+        for (auto const & iter : weights_cands)
+            DEBUG("Candidate " << g_.int_id(iter.first.e_) << " weight " << iter.second);
+
+        auto sort_res = MapToSortVector(weights_cands);
+        DEBUG("sort res " << sort_res.size() << " tr " << weight_priority_threshold_);
+        if (sort_res.empty() || sort_res[0].second < filtering_threshold_) {
+            filtered_cands.clear();
+        } else if (sort_res.size() > 1) {
+            if (sort_res[0].second > weight_priority_threshold_ * sort_res[1].second) {
+                filtered_cands.clear();
+                filtered_cands.insert(sort_res[0].first);
+            } else {
+                for (size_t i = 0; i < sort_res.size(); ++i) {
+                    if (sort_res[i].second * weight_priority_threshold_ < sort_res[0].second)
+                        filtered_cands.erase(sort_res[i].first);
+                }
+            }
+        }
+        EdgeContainer result(filtered_cands.begin(), filtered_cands.end());
+        if (result.size() != 1)
+            DEBUG("Trusted contigs don't help =(");
+        return result;
+    }
+
+private:
+
+    /// @return (there are covering paths that contains 'path[start_pos]')
+    std::pair<bool, std::set<EdgeWithDistance>> GetCandidates(const BidirectionalPath &path,
+                                             std::map<EdgeWithDistance, double> &weights_cands,
+                                             size_t start_pos,
+                                             const std::function<std::pair<bool, size_t>(const BidirectionalPath&, size_t)> &comparator) const
+    {
+        VERIFY(start_pos < path.Size());
+        std::set<EdgeWithDistance> filtered_cands;
+        auto support_paths = cov_map_.GetCoveringPaths(path[start_pos]);
+        DEBUG("Found " << support_paths.size() << " covering paths!!!");
+        for (auto const & it : support_paths) {
+            for (auto pos : it->FindAll(path[start_pos])) {
+                if (pos + 1 < it->Size()) {
+                    auto tmp = comparator(*it, pos);
+                    auto& is_good_path = tmp.first;
+                    auto& matched_len = tmp.second;
+                    if (is_good_path) {
+                        auto gap = it->GapAt(pos + 1);
+                        EdgeWithDistance next = {it->At(pos + 1), gap.gap, std::move(gap.gap_seq)};
+                        weights_cands[next] += static_cast<double>(matched_len)*it->GetWeight();
+                        filtered_cands.insert(next);
+                    }
+                }
+            }
+        }
+        return {!support_paths.empty(), std::move(filtered_cands)};
+    }
+
+    std::set<EdgeWithDistance> GetHighQualityCandidats(const BidirectionalPath &path, std::map<EdgeWithDistance, double> &weights_cands) const {
+        auto start_pos = path.Size() - 1;
+        auto comparator = [&path, start_pos, th = this] (const BidirectionalPath &coverage_path, size_t pos) {
+            auto is_good_path = (FirstNotEqualPosition(path, start_pos, coverage_path, pos, false) == -1ul);
+            auto matched_len = start_pos + 1;
+            auto privilege_scalar = (th->HasUniqueEdge(path, 0, matched_len) ? 3 : 1);
+            return std::pair<bool, size_t>(is_good_path, matched_len * privilege_scalar);
+        };
+        return GetCandidates(path, weights_cands, start_pos, comparator).second;
+    }
+
+    std::set<EdgeWithDistance> GetLowQualityCandidats(const BidirectionalPath &path, std::map<EdgeWithDistance, double> &weights_cands) const {
+        DEBUG("Fallback mode");
+        auto get_comparator = [&path, th = this](size_t start_pos){
+            /// compares the two path prefixes from 'start_pos', might skip several nonunique edges
+            return [&path, start_pos, th] (const BidirectionalPath &coverage_path, size_t pos) {
+                auto pos1 = static_cast<int>(start_pos);
+                auto pos2 = static_cast<int>(pos);
+                auto skipped_non_unique_edges = 0;
+                while (pos1 >= 0 && pos2 >= 0) {
+                    if (path[pos1] == coverage_path[pos2]) {
+                        skipped_non_unique_edges = 0;
+                        --pos2;
+                    } else {
+                        ++skipped_non_unique_edges;
+                        if (th->IsUniqueEdge(path[pos1]) || skipped_non_unique_edges > 0)
+                            break;
+                    }
+                    --pos1;
+                }
+                auto matched_len = pos-pos2;
+                auto is_good_path = th->HasUniqueEdge(coverage_path, pos2+1, matched_len);
+                return std::pair<bool, size_t>(is_good_path, matched_len);
+            };
+        };
+        bool used_unique_edge = false;
+        for (int i = 0; i < std::min((int)path.Size(), 2) && !used_unique_edge; ++i) {
+            DEBUG("iteration: " << i);
+            auto pos = path.Size() - 1 - i;
+            used_unique_edge = IsUniqueEdge(path[pos]);
+            auto tmp = GetCandidates(path, weights_cands, pos, get_comparator(pos));
+            auto& coverage_paths_are_found = tmp.first;
+            auto& filtered_cands = tmp.second;
+            if (!filtered_cands.empty() || coverage_paths_are_found)
+                return std::move(filtered_cands);
+        }
+        DEBUG("Fallback mode does not help");
+        return {};
+    }
+
+    bool IsUniqueEdge(EdgeId edge) const {
+        return unique_edge_analyzer_.IsUnique(edge) && g_.length(edge) >= min_significant_overlap_;
+    }
+
+    bool HasUniqueEdge(const BidirectionalPath& path, size_t from, size_t len) const {
+        for (size_t i = 0; i < len; ++i) {
+            auto edge = path.At(from + i);
+            if (IsUniqueEdge(edge))
+                return true;
+        }
+        return false;
+    }
+
+    std::vector<std::pair<EdgeWithDistance, double>> MapToSortVector(const std::map<EdgeWithDistance, double>& map) const {
+        std::vector<std::pair<EdgeWithDistance, double>> result(map.begin(), map.end());
+        std::sort(result.begin(), result.end(), EdgeWithWeightCompareReverse<EdgeWithDistance>);
+        return result;
+    }
+
+    double filtering_threshold_;
+    double weight_priority_threshold_;
+    size_t min_significant_overlap_;
+    const GraphCoverageMap& cov_map_;
+    LongReadsUniqueEdgeAnalyzer unique_edge_analyzer_;
+    bool use_low_quality_matching_;
+
+    DECL_LOGGER("TrustedContigsExtensionChooser");
+};
 
 class CoordinatedCoverageExtensionChooser: public ExtensionChooser {
 public:
-    CoordinatedCoverageExtensionChooser(const Graph& g, 
-            CoverageAwareIdealInfoProvider& coverage_provider, 
+    CoordinatedCoverageExtensionChooser(const Graph& g,
+            CoverageAwareIdealInfoProvider& coverage_provider,
             size_t max_edge_length_in_repeat, double delta, size_t min_path_len) :
-            ExtensionChooser(g), provider_(coverage_provider), 
+            ExtensionChooser(g), provider_(coverage_provider),
             max_edge_length_in_repeat_(max_edge_length_in_repeat), delta_(delta), min_path_len_(min_path_len) {
     }
 
@@ -1162,7 +1370,7 @@ public:
         }
         DEBUG("Path coverage is " << path_coverage);
 
-        for (auto e_d : edges) {
+        for (const auto& e_d : edges) {
             if (path.Contains(g_.EdgeEnd(e_d.e_))) {
                 DEBUG("Avoid to create loops");
                 return EdgeContainer();
@@ -1186,8 +1394,8 @@ private:
         }
     }
 
-    GraphComponent<Graph> GetRepeatComponent(const VertexId start, double path_coverage) const {
-        set<VertexId> vertices_of_component;
+    omnigraph::GraphComponent<Graph> GetRepeatComponent(const VertexId start, double path_coverage) const {
+        std::set<VertexId> vertices_of_component;
         vertices_of_component.insert(start);
         std::queue<VertexId> can_be_processed;
         UpdateCanBeProcessed(start, can_be_processed, path_coverage);
@@ -1196,20 +1404,20 @@ private:
             can_be_processed.pop();
             if (vertices_of_component.count(v) != 0) {
                 DEBUG("Component is too complex");
-                return GraphComponent<Graph>::Empty(g_);
+                return omnigraph::GraphComponent<Graph>::Empty(g_);
             }
             DEBUG("Adding vertex " << g_.str(v) << " to component set");
             vertices_of_component.insert(v);
             UpdateCanBeProcessed(v, can_be_processed, path_coverage);
         }
 
-        return GraphComponent<Graph>::FromVertices(g_, vertices_of_component);
+        return omnigraph::GraphComponent<Graph>::FromVertices(g_, vertices_of_component);
     }
 
     EdgeContainer FinalFilter(const EdgeContainer& edges,
             EdgeId edge_to_extend) const {
         EdgeContainer result;
-        for (auto e_with_d : edges) {
+        for (const auto& e_with_d : edges) {
             if (e_with_d.e_ == edge_to_extend) {
                 result.push_back(e_with_d);
             }
@@ -1235,10 +1443,10 @@ private:
         if (g_.length(ext) > max_edge_length_in_repeat_) {
             DEBUG("Long extension");
             return g_.coverage(ext);
-        } 
+        }
 
         DEBUG("Short extension, launching repeat component analysis");
-        GraphComponent<Graph> gc = GetRepeatComponent(g_.EdgeEnd(ext), path_coverage);
+        auto gc = GetRepeatComponent(g_.EdgeEnd(ext), path_coverage);
         if (gc.v_size() == 0) {
             DEBUG("Component search failed");
             return -1.;
@@ -1254,7 +1462,7 @@ private:
         DEBUG("Checking long sinks");
         for (auto v : gc.exits()) {
             for (auto e : g_.OutgoingEdges(v)) {
-                if (g_.length(e) > max_edge_length_in_repeat_ && 
+                if (g_.length(e) > max_edge_length_in_repeat_ &&
                         CompatibleEdge(e, path_coverage) &&
                         math::ls(g_.coverage(e), answer)) {
                     DEBUG("Updating answer to coverage of edge " << g_.str(e));
@@ -1269,9 +1477,9 @@ private:
     EdgeContainer FindExtensionTroughRepeat(const EdgeContainer& edges, double path_coverage) const {
         static EdgeContainer EMPTY_CONTAINER;
 
-        map<EdgeId, double> good_extension_to_ahead_cov;
+        std::map<EdgeId, double> good_extension_to_ahead_cov;
 
-        for (auto edge : edges) {
+        for (const auto& edge : edges) {
             DEBUG("Processing candidate extension " << g_.str(edge.e_));
             double analysis_res = AnalyzeExtension(edge.e_, path_coverage);
 
@@ -1303,7 +1511,7 @@ private:
 
         return EMPTY_CONTAINER;
     }
-    
+
     CoverageAwareIdealInfoProvider provider_;
     const size_t max_edge_length_in_repeat_;
     const double delta_;
