@@ -17,7 +17,7 @@ class DistanceEstimation : public spades::AssemblyStage {
         : AssemblyStage(preliminary ? "Preliminary Distance Estimation" : "Distance Estimation",
                         preliminary ? "distance_estimation_preliminary" : "distance_estimation") {}
 
-    void run(conj_graph_pack &gp, const char*);
+    void run(GraphPack &gp, const char*) override;
 };
 
 }
